﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel.Design;
using System.Runtime.InteropServices;

namespace Eka.Inheritance.MDIControls {
    /// <summary>
    /// Class to crate collapsible panel control.
    /// </summary>
    [DesignTimeVisible(true)]
    [Category("Containers")]
    [Description("Eka Collapsible Panel")]
    [Designer("System.Windows.Forms.Design.ParentControlDesigner, System.Design", typeof(IDesigner))]
    public partial class CollapsiblePanel : UserControl {
        #region Member Variables
        /// <summary>
        /// Event for value changed.
        /// </summary>
        public event EkaSelectedValueChangedEventHandler EkaSelectedValueChanged;

        /// <summary>
        /// Delegate for event. EkaSelectedValueChangedEventHandler.
        /// </summary>
        public delegate void EkaSelectedValueChangedEventHandler();

        // A delegate type for hooking up change notifications
        /// <summary>
        /// delegate for ToggleStateChanged event.
        /// </summary>
        /// <param name="eventArgs"></param>
        public delegate void ToggleChangeHandler(ToggleEventArgs eventArgs);

        /// An event that clients can use to be notified whenever the
        /// elements of the list change.
        /// <summary>
        /// Event to handle the Control on toggle state change.
        /// </summary>
        public event ToggleChangeHandler ToggleStateChanged;

        /// <summary>
        /// constant to define the Gap(no of pixel) between label text and image icon
        /// </summary>
        private const int mGapBetweenLabelAndToggleButton = 1;

        /// <summary>
        /// Member variable to define the gap (no of pixel) in between panels when 
        /// panels are arranged sequentially by Setting NextPanel properties
        /// </summary>
        private short mGapBetweenPanels = 1;

        /// <summary>
        /// Member variables to hoods Collapsing toggle state
        /// </summary>
        private ToggleButtonState mToggleButtonState = ToggleButtonState.Expanded;

        /// <summary>
        /// Member variable to holds the Collapsing button position
        /// </summary>
        private CollasibleButtonPosition mToggleButtonPosition = CollasibleButtonPosition.Right;

        /// <summary>
        /// Member variable to holds the control height in collapse/expand state
        /// </summary>
        private Size mControlsSize = new Size();

        /// <summary>
        /// Panel to be located beneath this panel
        /// </summary>
        CollapsiblePanel mNextPanel = null;

        /// <summary>
        /// Member variable to holds the Controls Border color (#666666)
        /// </summary>
        private Color mBorderColor = Color.LightGray;

        /// <summary>
        /// Member variable to holds the Border thickness properties. by default it will be zero(0).
        /// </summary>
        private short mBorederThickness = 1;

        /// <summary>
        /// Member variable to holds the border style of the control.
        /// Default: None
        /// </summary>
        private ButtonBorderStyle mButtonBorderStyle = ButtonBorderStyle.Solid;

        /// <summary>
        /// Member variable to holds the Draw border value
        /// </summary>
        private bool mDrawBorder = true;

        private const int TV_FIRST = 0x1100;
        private const int TVM_SETBKCOLOR = TV_FIRST + 29;
        private const int TVM_SETEXTENDEDSTYLE = TV_FIRST + 44;
        private const int TVS_EX_DOUBLEBUFFER = 0x0004;

        /// <summary>
        /// Win API to set DoubleBuffered = true for the tree view to avoid flickering
        /// </summary>
        /// <param name="hWnd"></param>
        /// <param name="msg"></param>
        /// <param name="wParam"></param>
        /// <param name="lParam"></param>
        /// <returns></returns>
        [DllImport("user32.dll")]
        public static extern IntPtr SendMessage(IntPtr hWnd, int msg, IntPtr wParam, IntPtr lParam);

        #region for Control Resizing and moving
        /// <summary>
        /// 
        /// </summary>
        private ResizeOptions mPanelResizeOption = new ResizeOptions(false, false, false, false, false, false);
        /// <summary>
        /// Member variable to track control is moving or not
        /// </summary>
        private bool mMoving;
        /// <summary>
        /// member variable to track the cursor Starting point
        /// </summary>
        private Point mCursorStartPoint;
        /// <summary>
        /// Member variable to track, that the mouse move is internal
        /// </summary>
        private bool mMoveIsInternal;
        /// <summary>
        /// Member variable to track  the control is resizing
        /// </summary>
        private bool mResizing;
        /// <summary>
        /// Member variable to track the control starting size
        /// </summary>
        private Size mControlStartSize;
        #endregion

        #endregion

        #region Constructor
        /// <summary>
        /// constructor method to instantiate the control.
        /// </summary>
        public CollapsiblePanel() {
            InitializeComponent();

            // Enable default double buffering processing
            SetStyle(ControlStyles.OptimizedDoubleBuffer | ControlStyles.AllPaintingInWmPaint, true);

            // Disable default CommCtrl painting on non-Vista systems
            if (!IsWinVista)
                SetStyle(ControlStyles.UserPaint, true);

            //Initialize default controls properties
            InitializeDefaultControlsTheme();
            this.Resize += CollapsiblePanel_Resize;
        } 
        #endregion

        #region Properties
        internal bool MouseIsInLeftEdge { get; set; }
        internal bool MouseIsInRightEdge { get; set; }
        internal bool MouseIsInTopEdge { get; set; }
        internal bool MouseIsInBottomEdge { get; set; }
        internal ControlOperation OperationType { get; set; }

        /// <summary>
        /// Get properties to get the minimum size of the control for resizing.
        /// </summary>
        public Size AllowedMinimumSize {
            get {
                return new Size(ctrToggleButtonPicturebox.Width + 2, ctrToggleButtonPicturebox.Height + 2); 
            }
        }
        /// <summary>
        /// Get/Set properties for panel resize options.
        /// </summary>
        [Category("Collapsible Panel")]
        [Description("Gets or set properties for panel resizing.")]
        public ResizeOptions PanelResizeOption {
            get {
                return this.mPanelResizeOption;
            }
            set {
                this.mPanelResizeOption = value;
            }
        }

        /// <summary>
        /// Get/Set properties to allow the panel to resize.
        /// </summary>
        public bool AllowResize {
            get;
            set;
        }

        /// <summary>
        /// Get/Set properties for Panel header text
        /// </summary>
        [Category("Collapsible Panel")]
        [Description("Gets or set properties for button toggle state")]
        public ToggleButtonState ToggleButtonState {
            get {
                return this.mToggleButtonState;
            }

            set {
                if (value == this.mToggleButtonState)
                    return;
                OnMouseClickOnToggleButton();
            }
        }

        /// <summary>
        /// Get/Set properties for Panel header text
        /// </summary>
        [Category("Collapsible Panel")]
        [Description("Gets or set properties for Panel header text")]
        public string PanelHeaderText {
            get {
                return ctrHeaderLabel.Text;
            }

            set {
                ctrHeaderLabel.Text = value;
            }
        }

        /// <summary>
        /// Get/Set properties for Panel Header Height
        /// </summary>
        [Category("Collapsible Panel")]
        [Description("Gets or sets properties for Panel Header Height")]
        public int PanelHeaderHeight {
            get {
                return ctrHeaderPanel.Height;
            }
            set {
                ctrHeaderPanel.Height = value;
                RepositionControls();
            }
        }

        /// <summary>
        /// Get/Set properties for Panel Header text alignment
        /// </summary>
        [Category("Collapsible Panel")]
        [Description("Gets or sets properties for Panel Header text alignment")]
        public ContentAlignment PanelHeaderTextAligh {
            get { return ctrHeaderLabel.TextAlign; }
            set { ctrHeaderLabel.TextAlign = value; }
        }

        /// <summary>
        /// Get/Set properties for Panel header Background color
        /// </summary>
        [Category("Collapsible Panel")]
        [Description("Gets or sets properties for Panel header Background color")]
        public Color PanelHeaderBackGroundColor {
            get { 
                return ctrHeaderPanel.BackColor; 
            }

            set {
                ctrHeaderPanel.BackColor = value;
                ctrHeaderLabel.BackColor = value;
                ctrToggleButtonPicturebox.BackColor = value;
            }
        }

        /// <summary>
        /// Set properties for the control Height
        /// </summary>
        [Category("Collapsible Panel")]
        [Description("Set properties for the control Height")]
        public int SetHeight {
            get {
                return this.Height;
            }

            set {
                this.Height = value;
                MoveToNextPanel();
                Invalidate();
            }
        }

        /// <summary>
        /// Gets or sets the Gap between the panel located beneath this panel
        /// </summary>
        [Category("Collapsible Panel")]
        [Description("Gets or sets the Gap between the panel located beneath this panel")]
        public short GapBetweenPanels {
            get {
                return mGapBetweenPanels;
            }

            set {
                mGapBetweenPanels = value;
                MoveToNextPanel();
            }
        }

        /// <summary>
        /// Get/Set properties for Collapsible button Position
        /// </summary>
        [Category("Collapsible Panel")]
        [Description("Gets or sets properties for Collapsible button Position")]
        public CollasibleButtonPosition ToggleButtonPosition {
            get {
                return mToggleButtonPosition;
            }

            set {
                if (value == mToggleButtonPosition)
                    return;

                mToggleButtonPosition = value;
                RepositionControls();
            }
        }

        /// <summary>
        /// Gets or sets the panel to be located beneath this panel
        /// </summary>
        [Category("Collapsible Panel")]
        [Description("Gets or sets the panel to be located beneath this panel")]
        public CollapsiblePanel NextPanel {
            get {
                return mNextPanel;
            }

            set {
                mNextPanel = value;
                MoveToNextPanel();
            }
        }

        /// <summary>
        /// Get/Set the border color
        /// </summary>
        [Category("Collapsible Panel")]
        [Description("Get/Set the border color")]
        public Color BorderColor {
            get {
                return mBorderColor;
            }

            set {
                mBorderColor = value;
                Invalidate();
            }
        }

        /// <summary>
        /// Get/Set Property to indicate the border to be drawn or not.
        /// </summary>
        [Category("Collapsible Panel")]
        [Description("Property to indicate the border to be drawn or not.")]
        public bool DrawBorder {
            get {
                return mDrawBorder;
            }

            set {
                mDrawBorder = value;
                Invalidate();
            }
        }
        #endregion
        
        #region overridden method
        /// <summary>
        /// overridden method for control onHandlecreated.
        /// </summary>
        /// <param name="e"></param>
        protected override void OnHandleCreated(EventArgs e) {
            base.OnHandleCreated(e);


            UpdateExtendedStyles();
            if (!IsWinXP)
                SendMessage(Handle, TVM_SETBKCOLOR, IntPtr.Zero, (IntPtr)ColorTranslator.ToWin32(BackColor));
        }

        /// <summary>
        /// overridden method for OnFontChanged event.
        /// </summary>
        /// <param name="e"></param>
        protected override void OnFontChanged(EventArgs e) {
            base.OnFontChanged(e);
            ctrHeaderLabel.Font = this.Font;
            Invalidate();
        }

        /// <summary>
        /// overridden method for OnClientSizeChanged event.
        /// </summary>
        /// <param name="e"></param>
        protected override void OnClientSizeChanged(EventArgs e) {
            base.OnClientSizeChanged(e);
            this.Invalidate();
        }

        /// <summary>
        /// Paint method for drawing the Border line
        /// </summary>
        /// <param name="e"></param>
        protected override void OnPaint(PaintEventArgs e) {
            base.OnPaint(e);

            if (mDrawBorder) {
                switch (mToggleButtonState) {
                    case ToggleButtonState.Expanded:
                        ControlPaint.DrawBorder(e.Graphics, ClientRectangle,
                                    mBorderColor, mBorederThickness, mButtonBorderStyle,
                                    mBorderColor, mBorederThickness, mButtonBorderStyle,
                                    mBorderColor, mBorederThickness, mButtonBorderStyle,
                                    mBorderColor, mBorederThickness, mButtonBorderStyle);
                        break;

                    case ToggleButtonState.Collapsed:

                        ControlPaint.DrawBorder(e.Graphics, new Rectangle(new Point(0, 0), new Size(this.Width, this.Height)),
                                    mBorderColor, mBorederThickness, mButtonBorderStyle,
                                    mBorderColor, mBorederThickness, mButtonBorderStyle,
                                    mBorderColor, mBorederThickness, mButtonBorderStyle,
                                    mBorderColor, mBorederThickness, mButtonBorderStyle);
                        break;
                }
            }
        }
        #endregion

        #region Private Member Function
        private void CollapsiblePanel_Resize(object sender, EventArgs e) {
            MoveToNextPanel();
        }

        internal void Register(Control control) {
            Register(control, control);
        }

        internal void Register(Control control, Control container) {
            mMoving = false;
            mResizing = false;
            mMoveIsInternal = false;
            mCursorStartPoint = Point.Empty;
            MouseIsInLeftEdge = false;
            MouseIsInRightEdge = false;
            MouseIsInTopEdge = false;
            MouseIsInBottomEdge = false;
            OperationType = ControlOperation.Resize;
            control.MouseDown += (sender, e) => StartMovingOrResizing(control, e);
            control.MouseUp += (sender, e) => StopDragOrResizing(control);
            control.MouseMove += (sender, e) => MoveControl(container, e);
        }

        private void UpdateMouseEdgeProperties(Control control, Point mouseLocationInControl) {
            if (mToggleButtonState == ToggleButtonState.Collapsed || !AllowResize)
                return;

            if (OperationType == ControlOperation.Move)
                return;

            MouseIsInLeftEdge = Math.Abs(mouseLocationInControl.X) <= 3;
            MouseIsInRightEdge = Math.Abs(mouseLocationInControl.X - control.Width) <= 3;
            MouseIsInTopEdge = Math.Abs(mouseLocationInControl.Y) <= 3;
            MouseIsInBottomEdge = Math.Abs(mouseLocationInControl.Y - control.Height) <= 3;
        }

        private void UpdateMouseCursor(Control lControl) {
            if (mToggleButtonState == ToggleButtonState.Collapsed || !AllowResize)
                return;

            if (OperationType == ControlOperation.Move)
                return;

            if (MouseIsInLeftEdge) {
                lControl.Cursor = GeLeftEdgeMouseCursor();
            }
            else if (MouseIsInRightEdge) {
                lControl.Cursor = GeRightEdgeMouseCursor();
            }
            else if (MouseIsInTopEdge) {
                lControl.Cursor = GetTopEdgeCursor();
            }
            else if (MouseIsInBottomEdge) {
                lControl.Cursor = GetBottomEdgeCursor();
            }
            else {
                //Finally it is the normal cursor
                lControl.Cursor = Cursors.Default;
            }
        }

        private Cursor GeLeftEdgeMouseCursor() {
            Cursor lLeftEdgeCursor = Cursors.Default;

            //return normal cursor if allow resize is not enabled
            if (!AllowResize)
                return lLeftEdgeCursor;

            //return normal cursor if resizing is not allowed from left
            if (!PanelResizeOption.AllowFromLeft)
                return lLeftEdgeCursor;

            //return normal cursor if mouse in not in Left Edge
            if (!MouseIsInLeftEdge)
                return lLeftEdgeCursor;

            //Now the mouse is in left edge, we need to verify the corner edge of left side
            if (MouseIsInTopEdge && PanelResizeOption.AllowResizeFromTopEdge) {
                lLeftEdgeCursor = Cursors.SizeNWSE;
            }
            else if (MouseIsInBottomEdge && PanelResizeOption.AllowResizeFromBottomEdge) {
                lLeftEdgeCursor = Cursors.SizeNESW;
            }
            else {
                lLeftEdgeCursor = Cursors.SizeWE;
            }

            return lLeftEdgeCursor;
        }

        private Cursor GeRightEdgeMouseCursor() {
            Cursor lRightEdgeCursor = Cursors.Default;

            //return normal cursor if allow resize is not enabled
            if (!AllowResize)
                return lRightEdgeCursor;

            //return normal cursor if resizing is not allowed from left
            if (!PanelResizeOption.AllowFromRight)
                return lRightEdgeCursor;

            //return normal cursor if mouse in not in Left Edge
            if (!MouseIsInRightEdge)
                return lRightEdgeCursor;

            //Now the mouse is in left edge, we need to verify the corner edge of left side
            if (MouseIsInTopEdge && PanelResizeOption.AllowResizeFromTopEdge) {
                lRightEdgeCursor = Cursors.SizeNESW;
            }
            else if (MouseIsInBottomEdge && PanelResizeOption.AllowResizeFromBottomEdge) {
                lRightEdgeCursor = Cursors.SizeNWSE;
            }
            else {
                lRightEdgeCursor = Cursors.SizeWE;
            }

            return lRightEdgeCursor;
        }

        private Cursor GetTopEdgeCursor() {
            Cursor lTopEdgeCursor = Cursors.Default;
            //Check the mouse is in top edge or not
            if (!MouseIsInTopEdge)
                return lTopEdgeCursor;

            //return normal cursor if allow resize is not enabled
            if (!AllowResize)
                return lTopEdgeCursor;

            //return normal cursor if resizing is not allowed from left
            if (!PanelResizeOption.AllowFromTop)
                return lTopEdgeCursor;

            //Now resize is allowed from top, return the resize cursor for top edge
            lTopEdgeCursor = Cursors.SizeNS;

            return lTopEdgeCursor;
        }

        private Cursor GetBottomEdgeCursor() {
            Cursor lBottomEdgeCursor = Cursors.Default;

            //Check the mouse is in top edge or not
            if (!MouseIsInBottomEdge)
                return lBottomEdgeCursor;

            //return normal cursor if allow resize is not enabled
            if (!AllowResize)
                return lBottomEdgeCursor;

            //return normal cursor if resizing is not allowed from left
            if (!PanelResizeOption.AllowFromBottom)
                return lBottomEdgeCursor;

            //Now resize is allowed from top, return the resize cursor for top edge
            lBottomEdgeCursor = Cursors.SizeNS;

            return lBottomEdgeCursor;
        }

        private void StartMovingOrResizing(Control lControl, MouseEventArgs lEvent) {
            if (mToggleButtonState == ToggleButtonState.Collapsed || !AllowResize)
                return;

            if (mMoving || mResizing)
                return;

            if (OperationType != ControlOperation.Move &&
                (MouseIsInRightEdge || MouseIsInLeftEdge || MouseIsInTopEdge || MouseIsInBottomEdge)) {
                mResizing = true;
                mControlStartSize = lControl.Size;
            }
            else if (OperationType != ControlOperation.Resize) {
                mMoving = true;
                lControl.Cursor = Cursors.Hand;
            }
            mCursorStartPoint = new Point(lEvent.X, lEvent.Y);
            lControl.Capture = true;
        }

        private void MoveControl(Control lControl, MouseEventArgs lEvent) {
            if (mToggleButtonState == ToggleButtonState.Collapsed || !AllowResize) {
                lControl.Cursor = Cursors.Default;
                return;
            }

            if (!mResizing && !mMoving) {
                UpdateMouseEdgeProperties(lControl, new Point(lEvent.X, lEvent.Y));
                UpdateMouseCursor(lControl);
            }

            //do not allow to do resize if the cursor is in normal state
            if (lControl.Cursor == Cursors.Default)
                return;

            if (mResizing) {
                if (MouseIsInLeftEdge) {
                    if (MouseIsInTopEdge) {
                        lControl.Width -= (lEvent.X - mCursorStartPoint.X);
                        lControl.Left += (lEvent.X - mCursorStartPoint.X);
                        lControl.Height -= (lEvent.Y - mCursorStartPoint.Y);
                        lControl.Top += (lEvent.Y - mCursorStartPoint.Y);
                    }
                    else if (MouseIsInBottomEdge) {
                        lControl.Width -= (lEvent.X - mCursorStartPoint.X);
                        lControl.Left += (lEvent.X - mCursorStartPoint.X);
                        lControl.Height = (lEvent.Y - mCursorStartPoint.Y) + mControlStartSize.Height;
                    }
                    else {
                        lControl.Width -= (lEvent.X - mCursorStartPoint.X);
                        lControl.Left += (lEvent.X - mCursorStartPoint.X);
                    }
                }
                if (MouseIsInRightEdge) {
                    if (MouseIsInTopEdge) {
                        lControl.Width = (lEvent.X - mCursorStartPoint.X) + mControlStartSize.Width;
                        lControl.Height -= (lEvent.Y - mCursorStartPoint.Y);
                        lControl.Top += (lEvent.Y - mCursorStartPoint.Y);

                    }
                    else if (MouseIsInBottomEdge) {
                        lControl.Width = (lEvent.X - mCursorStartPoint.X) + mControlStartSize.Width;
                        lControl.Height = (lEvent.Y - mCursorStartPoint.Y) + mControlStartSize.Height;
                    }
                    else {
                        lControl.Width = (lEvent.X - mCursorStartPoint.X) + mControlStartSize.Width;
                    }
                }
                else if (MouseIsInTopEdge) {
                    lControl.Height -= (lEvent.Y - mCursorStartPoint.Y);
                    lControl.Top += (lEvent.Y - mCursorStartPoint.Y);
                }
                else if (MouseIsInBottomEdge) {
                    lControl.Height = (lEvent.Y - mCursorStartPoint.Y) + mControlStartSize.Height;
                }
                else {
                    StopDragOrResizing(lControl);
                }
            }
            else if (mMoving) {
                mMoveIsInternal = !mMoveIsInternal;
                if (!mMoveIsInternal) {
                    int x = (lEvent.X - mCursorStartPoint.X) + lControl.Left;
                    int y = (lEvent.Y - mCursorStartPoint.Y) + lControl.Top;
                    lControl.Location = new Point(x, y);
                }
            }
        }

        private void StopDragOrResizing(Control lControl) {
            if (mToggleButtonState == ToggleButtonState.Collapsed || !AllowResize)
                return;

            mResizing = false;
            mMoving = false;
            lControl.Capture = false;
            UpdateMouseCursor(lControl);
        }

        /// <summary>
        /// Method to initialize the control initial theme
        /// </summary>
        private void InitializeDefaultControlsTheme() {
            //Register for resizing
            Register(this);

            ctrHeaderPanel.TabStop = false;
            ctrHeaderLabel.Font = new System.Drawing.Font("Microsoft Sans Serif", 8F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            ctrHeaderLabel.Height = 21;
            ctrHeaderLabel.AutoSize = false;

            base.Font = new System.Drawing.Font("Microsoft Sans Serif", 8F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));

            //initialize toggle button position
            mToggleButtonPosition = CollasibleButtonPosition.Right;

            //initialize toggle button state
            mToggleButtonState = ToggleButtonState.Expanded;

            //Set the default properties of the Controls
            Size = new Size(300, 150);
            BackColor = Color.White;
            BorderStyle = BorderStyle.None;
            Padding = new Padding(1, 1, 1, 1);
            this.LocationChanged += OnControlLocationChanged;
            mControlsSize = this.Size;

            //Set the default properties for Panel Header
            ctrHeaderPanel.Dock = DockStyle.Top;
            ctrHeaderPanel.Height = 21;
            ctrHeaderPanel.BackColor = Color.White;
            ctrHeaderPanel.BorderStyle = BorderStyle.None;
            ctrHeaderPanel.Padding = new Padding(0, 0, 0, 0);
            ctrHeaderLabel.Text = "Panel Header";
            ctrHeaderPanel.Resize += OnPanelHeaderSizeChanged;
            ctrHeaderPanel.Location = new Point(0, 0);

            //Set the image button style
            ctrToggleButtonPicturebox.BackColor = ctrHeaderPanel.BackColor;
            ctrToggleButtonPicturebox.SizeMode = PictureBoxSizeMode.CenterImage;
            BindImagesToToggleButton();
            ctrToggleButtonPicturebox.MouseHover += OnMouseHoverAndEnterToPictureBox;
            ctrToggleButtonPicturebox.MouseEnter += OnMouseHoverAndEnterToPictureBox;
            ctrToggleButtonPicturebox.MouseLeave += OnToggleButtonClick;
            ctrToggleButtonPicturebox.MouseClick += ctrToggleButtonPicturebox_MouseClick;
            //Position controls according to the setting
            RepositionControls();
        }

        /// <summary>
        /// This method will execute when the control position change
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnControlLocationChanged(object sender, EventArgs e) {
            MoveToNextPanel();
        }

        /// <summary>
        /// Method to resize the control panel when the controls toggle state change.
        /// </summary>
        private void ResizeControlOnToggleButtonStateChange() {
            //First cache the current control height
            switch (mToggleButtonState) {
                case ToggleButtonState.Expanded:
                    this.Height = mControlsSize.Height;
                    break;

                case ToggleButtonState.Collapsed:
                    mControlsSize = this.Size;
                    this.Height = ctrHeaderPanel.Height + ctrHeaderPanel.Padding.Top + ctrHeaderPanel.Padding.Bottom + 2;// +5;// - 2;
                    break;
            }
        }

        /// <summary>
        /// This method will execute when mouse click will happen on Toggle Image button.
        /// which will change the toggle state and image of the control
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ctrToggleButtonPicturebox_MouseClick(object sender, MouseEventArgs e) {
            OnMouseClickOnToggleButton();
            Invalidate();
        }

        private void OnMouseClickOnToggleButton() {
            switch (mToggleButtonState) {
                case ToggleButtonState.Expanded:
                    this.mToggleButtonState = ToggleButtonState.Collapsed;
                    break;

                case ToggleButtonState.Collapsed:
                    this.mToggleButtonState = ToggleButtonState.Expanded;
                    break;
            }

            //First bind the toggle images
            BindImagesToToggleButton();

            //Resize the control height
            ResizeControlOnToggleButtonStateChange();
            MoveToNextPanel();
            RaiseToggleStateChanged();
        }

        /// <summary>
        /// Method to raise control toggle state change event.
        /// </summary>
        protected virtual void RaiseToggleStateChanged() {
            if (ToggleStateChanged != null) {
                ToggleEventArgs lToggleArgs = new ToggleEventArgs();
                lToggleArgs.CurrentToggleState = this.mToggleButtonState;

                ToggleStateChanged(lToggleArgs);
            }
        }

        /// <summary>
        /// Method will execute on toggle button click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnToggleButtonClick(object sender, EventArgs e) {
            BindImagesToToggleButton();
            MoveToNextPanel();
            Invalidate();
        }

        /// <summary>
        /// Method to bind images to toggle button according to the toggle state
        /// </summary>
        private void BindImagesToToggleButton() {
            switch (mToggleButtonState) {
                case ToggleButtonState.Expanded:
                    ctrToggleButtonPicturebox.Image = global::Eka.Inheritance.MDIControls.Properties.Resources.TopArrowWhite;
                    break;

                case ToggleButtonState.Collapsed:
                    ctrToggleButtonPicturebox.Image = global::Eka.Inheritance.MDIControls.Properties.Resources.BottomArrowWhite;
                    break;
            }
        }

        /// <summary>
        /// Moves the next panel down (when user control state is changed or the control is relocated)
        /// </summary>
        private void MoveToNextPanel() {
            if (mNextPanel != null) {
                mNextPanel.Location = new Point(mNextPanel.Location.X, this.Location.Y + this.Size.Height + mGapBetweenPanels);
            }
        }

        /// <summary>
        /// Method to execute on Header panel size change events
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnPanelHeaderSizeChanged(object sender, EventArgs e) {
            RepositionControls();
            MoveToNextPanel();
            Invalidate();
        }

        /// <summary>
        /// Method to calculate the position of all controls location and size
        /// </summary>
        private void RepositionControls() {
            ctrToggleButtonPicturebox.Width = ctrHeaderPanel.Height - ctrHeaderPanel.Padding.Top - ctrHeaderPanel.Padding.Bottom;// -2;
            ctrToggleButtonPicturebox.Height = ctrToggleButtonPicturebox.Width;

            int lHeaderWidth = 0;
            int lHeaderHeight = 0;
            lHeaderWidth = ctrHeaderPanel.Width - ctrHeaderPanel.Padding.Left - ctrHeaderPanel.Padding.Right - ctrToggleButtonPicturebox.Width - mGapBetweenLabelAndToggleButton;// -2;
            lHeaderHeight = ctrHeaderPanel.Height - ctrHeaderPanel.Padding.Top - ctrHeaderPanel.Padding.Bottom - 2;
            ctrHeaderLabel.Size = new System.Drawing.Size(lHeaderWidth, lHeaderHeight);

            switch (mToggleButtonPosition) {
                case CollasibleButtonPosition.Right:
                    ctrHeaderLabel.Location = new Point(ctrHeaderPanel.Padding.Left, ctrHeaderPanel.Padding.Top);
                    ctrToggleButtonPicturebox.Location = new Point(ctrHeaderLabel.Location.X + ctrHeaderLabel.Width + mGapBetweenLabelAndToggleButton + 1, ctrHeaderLabel.Location.Y);
                    break;

                case CollasibleButtonPosition.Left:
                    ctrToggleButtonPicturebox.Location = new Point(ctrHeaderPanel.Padding.Left, ctrHeaderPanel.Padding.Top);

                    ctrHeaderLabel.Location = new Point(ctrToggleButtonPicturebox.Location.X + ctrToggleButtonPicturebox.Width + mGapBetweenLabelAndToggleButton, ctrToggleButtonPicturebox.Location.Y);
                    break;
            }
        }

        #region Change images for Mouse Enter/Hover events

        private void OnMouseHoverAndEnterToPictureBox(object sender, EventArgs e) {
            BindMouseHoverAndEnterImageToButton();
        }

        private void BindMouseHoverAndEnterImageToButton() {
            switch (mToggleButtonState) {
                case ToggleButtonState.Expanded:
                    ctrToggleButtonPicturebox.Image = global::Eka.Inheritance.MDIControls.Properties.Resources.TopArrowOrange;
                    break;

                case ToggleButtonState.Collapsed:
                    ctrToggleButtonPicturebox.Image = global::Eka.Inheritance.MDIControls.Properties.Resources.BotArrowOrange;
                    break;
            }
        }
        #endregion

        private void DrawText(Graphics graphics) {
            graphics = this.CreateGraphics();
            string drawString = "Sample Text";
            System.Drawing.Font drawFont = new System.Drawing.Font("Microsoft Sans Serif", 16);
            System.Drawing.SolidBrush drawBrush = new System.Drawing.SolidBrush(System.Drawing.Color.Black);
            float x = 150.0f;
            float y = 50.0f;
            System.Drawing.StringFormat drawFormat = new System.Drawing.StringFormat(StringFormatFlags.DirectionVertical);
            graphics.DrawString(drawString, drawFont, drawBrush, x, y, drawFormat);
            drawFont.Dispose();
            drawBrush.Dispose();
            graphics.Dispose();
        }

        private void UpdateExtendedStyles() {
            int Style = 0;

            if (DoubleBuffered)
                Style |= TVS_EX_DOUBLEBUFFER;

            if (Style != 0)
                SendMessage(Handle, TVM_SETEXTENDEDSTYLE, (IntPtr)TVS_EX_DOUBLEBUFFER, (IntPtr)Style);
        }

        private static bool IsWinVista {
            get {
                OperatingSystem OS = Environment.OSVersion;
                return (OS.Platform == PlatformID.Win32NT) && (OS.Version.Major >= 6);
            }
        }

        private static bool IsWinXP {
            get {
                OperatingSystem OS = Environment.OSVersion;
                return (OS.Platform == PlatformID.Win32NT) &&
                    ((OS.Version.Major > 5) || ((OS.Version.Major == 5) && (OS.Version.Minor == 1)));
            }
        }
        #endregion     
    }
}