﻿using Microsoft.Win32.SafeHandles;
using System;
using System.ComponentModel;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using ComboBox = System.Windows.Forms.ComboBox;

namespace Eka.Inheritance.MDIControls.FormSearchBox
{
    /// <summary>
    /// This is a combobox with a suggestion list of items
    /// Searches are made against the pattern in the combo textbox by matching
    /// all the characters in the pattern in the right order but not consecutively
    /// </summary>
    public class AutoCompleComboBox : ComboBox
    {
        /// <summary>
        /// Define struct for COMBOBOXINFO
        /// </summary>
        [StructLayout(LayoutKind.Sequential)]
        public struct COMBOBOXINFO
        {
            /// <summary>
            /// cbSize
            /// </summary>
            public int CbSize;
            /// <summary>
            /// rcItem
            /// </summary>
            public RECT RItem;
            /// <summary>
            /// RcButton
            /// </summary>
            public RECT RcButton;
            /// <summary>
            /// StateButton
            /// </summary>
            public int StateButton;
            /// <summary>
            /// HwndCombo
            /// </summary>
            public IntPtr HwndCombo;
            /// <summary>
            /// HwndEdit
            /// </summary>
            public IntPtr HwndEdit;
            /// <summary>
            /// HwndList
            /// </summary>
            public IntPtr HwndList;
        }

        /// <summary>
        /// Define struct for RECT object
        /// </summary>
        [StructLayout(LayoutKind.Sequential)]
        public struct RECT
        {
            /// <summary>
            /// Left
            /// </summary>
            public int Left;
            /// <summary>
            /// Top
            /// </summary>
            public int Top;
            /// <summary>
            /// Right
            /// </summary>
            public int Right;
            /// <summary>
            /// Bottom
            /// </summary>
            public int Bottom;
        }

        #region Member variables
        /// <summary>
        /// PInvoke method to send message
        /// </summary>
        /// <param name="hWnd"></param>
        /// <param name="msg"></param>
        /// <param name="wp"></param>
        /// <param name="lp"></param>
        /// <returns></returns>
        [DllImport("user32.dll", EntryPoint = "SendMessageW")]
        private static extern IntPtr SendMessage(IntPtr hWnd, int msg, IntPtr wp, IntPtr lp);

        /// <summary>
        /// Api to draw floating text in a control
        /// </summary>
        /// <param name="hWnd"></param>
        /// <param name="pcbi"></param>
        /// <returns></returns>
        [DllImport("user32.dll")]
        public static extern bool GetComboBoxInfo(IntPtr hWnd, ref COMBOBOXINFO pcbi);

        /// <summary>
        /// Member variable to set the floating text
        /// </summary>
        private string mFloatingText = "Enter search text here";

        /// <summary>
        /// member variable for The Custom dropdown control
        /// </summary>
        private CustomDropDown mCustomDropDown;

        /// <summary>
        /// Member variable to set the suggestion list inside the drop down control
        /// </summary>
        private ListBox mSuggestionList;

        /// <summary>
        /// Member variable to set bold font for the typed text in the listbox
        /// </summary>
        private Font mBoldFont;

        /// <summary>
        /// Member variable to know if the last text change is triggered by the keyboad.
        /// </summary>
        private bool mFromKeyboard;

        /// <summary>
        /// Member variable for search matching type
        /// </summary>
        private SearchMatchingType mSearchMatchingType;

        /// <summary>
        /// Member variable to crate the instance for BorderDrawer to draw the control border.
        /// </summary>
        private BorderDrawer mBorderDrawer = null;

        /// <summary>
        /// Flag: Has Dispose already been called?
        /// </summary>
        bool mDisposed = false;

        /// <summary>
        /// Get/set Drop down font color
        /// </summary>
        private Color mDropDownForeColor = Color.Black;

        /// <summary>
        /// Instantiate a SafeHandle instance.
        /// </summary>
        SafeHandle mSafeHandle = new SafeFileHandle(IntPtr.Zero, true);

        /// <summary>
        /// A delegate type for hooking up change notifications.
        /// </summary>
        /// <param name="eventArgs"></param>
        public delegate void SearchTextHandler(SearchTextArgs eventArgs);

        /// <summary>
        /// An event that clients can use to be notified whenever the elements of the list change.
        /// </summary>
        public event SearchTextHandler SearchItemSelected;              
        #endregion

        #region Constructor
        /// <summary>
        /// constructor
        /// </summary>
        public AutoCompleComboBox()
        {
            //Initialize the control with basic themes
            SetBasicTheme();

            mBorderDrawer = new BorderDrawer();
            mBorderDrawer.BorderColor = Color.LightGray;
            mSearchMatchingType = SearchMatchingType.NoWildcard;

            // let's build our suggestion list
            mSuggestionList = new ListBox
            {
                DisplayMember = "Text",
                TabStop = false,
                Dock = DockStyle.Fill,
                DrawMode = DrawMode.OwnerDrawFixed,
                IntegralHeight = true,
                Sorted = Sorted,
            };

            mSuggestionList.Click += OnSuggestionListClick;
            mSuggestionList.DrawItem += OnSuggestionListDrawItem;
            FontChanged += OnFontChanged;
            mSuggestionList.MouseMove += OnSuggestionListMouseMove;
            mCustomDropDown = new CustomDropDown(mSuggestionList);
            OnFontChanged(null, null);
        }
        #endregion

        #region Properteis
        /// <summary>
        /// Get/Set Control Border coloer
        /// </summary>
        public Color BorderColor
        {
            get
            {
                return mBorderDrawer.BorderColor;
            }

            set
            {
                if (mBorderDrawer.BorderColor == value)
                    return;

                mBorderDrawer.BorderColor = value;
                Invalidate();
            }
        }

        /// <summary>
        /// Get/Set search matching type
        /// </summary>
        [DefaultValue(SearchMatchingType.NoWildcard)]
        [Description("How strings are matched against the user input")]
        [Browsable(true)]
        [EditorBrowsable(EditorBrowsableState.Always)]
        [Category("Behavior")]
        public SearchMatchingType MatchingType
        {
            get
            {
                return mSearchMatchingType;
            }

            set
            {
                if (mSearchMatchingType != value)
                {
                    mSearchMatchingType = value;
                    if (mCustomDropDown.Visible)
                    {
                        // recalculate the matches
                        ShowDropDown();
                    }
                }
            }
        }

        /// <summary>
        /// get/set sorted behaviour for items in the dropdown list
        /// </summary>
        [Category("Behavior"), DefaultValue(false)]
        [Description("Specifies whether items in the list portion of the combobo are sorted.")]
        public new bool Sorted
        {
            get
            {
                return base.Sorted;
            }

            set
            {
                mSuggestionList.Sorted = value;
                base.Sorted = value;
            }
        }

        /// <summary>
        /// Get/set Dropped down properties
        /// </summary>
        [Browsable(false)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public new bool DroppedDown
        {
            get
            {
                return base.DroppedDown || mCustomDropDown.Visible;
            }

            set
            {
                mCustomDropDown.Visible = false;
                base.DroppedDown = value;
            }
        }

        /// <summary>
        /// Get/Set Floating Text for the control
        /// </summary>
        public string FloatingText
        {
            get
            {
                return mFloatingText;
            }

            set
            {
                mFloatingText = value;
                UpdateFloatingText();
            }
        }

        #region Hidden inherited properties
        /// <summary>
        /// This property is not relevant for this class.
        /// </summary>
        [Browsable(false)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [EditorBrowsable(EditorBrowsableState.Never)]
        public new AutoCompleteSource AutoCompleteSource
        {
            get { return base.AutoCompleteSource; }
            set { base.AutoCompleteSource = value; }
        }

        /// <summary>
        /// This property is not relevant for this class.
        /// </summary>
        [Browsable(false)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [EditorBrowsable(EditorBrowsableState.Never)]
        public new AutoCompleteStringCollection AutoCompleteCustomSource
        {
            get { return base.AutoCompleteCustomSource; }
            set { base.AutoCompleteCustomSource = value; }
        }

        /// <summary>
        /// This property is not relevant for this class.
        /// </summary>
        [Browsable(false)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [EditorBrowsable(EditorBrowsableState.Never)]
        public new AutoCompleteMode AutoCompleteMode
        {
            get { return base.AutoCompleteMode; }
            set { base.AutoCompleteMode = value; }
        }

        /// <summary>
        /// This property is not relevant for this class.
        /// </summary>
        [Browsable(false)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [EditorBrowsable(EditorBrowsableState.Never)]
        public new ComboBoxStyle DropDownStyle
        {
            get { return base.DropDownStyle; }
            set { base.DropDownStyle = value; }
        }

        /// <summary>
        /// Get/Set value for drawing border. 
        /// True: draw border, which is default value.
        /// False: without border
        /// </summary>
        [DefaultValue(true)]
        public bool DrawBorder
        {
            get;
            set;
        }

       /// <summary>
       /// Get/Set Dropdown font color
       /// </summary>
        public Color DropDownForeColor
        {
            get
            {
                return mDropDownForeColor;
            }
            set
            {
                mDropDownForeColor = value;
            }
        }
        #endregion
        #endregion

        #region Public Method
        /// <summary>
        /// Shows the drop down.
        /// </summary>
        public void ShowDropDown()
        {
            //Dont do anything on design mode
            if (DesignMode)
                return;

            // Hide the "standard" drop down if any
            if (base.DroppedDown)
            {
                BeginUpdate();

                // setting DroppedDown to false may select an item
                // so we save the editbox state. just swap the text
                string lText = Text;
                int lSelStart = SelectionStart;
                int lSelLen = SelectionLength;

                // close the "standard" dropdown
                base.DroppedDown = false;

                // and restore the contents of the editbox
                Text = lText;
                Select(lSelStart, lSelLen);
                EndUpdate();
            }
            // pop it up and resize it
            int h = Math.Min(MaxDropDownItems, mSuggestionList.Items.Count) * mSuggestionList.ItemHeight;
            mCustomDropDown.Show(this, new Size(DropDownWidth, h));
        }

        /// <summary>
        /// Hides the drop down.
        /// </summary>
        public void HideDropDown()
        {
            if (mCustomDropDown.Visible)
            {
                mCustomDropDown.Close();
            }
        }
        #endregion

        #region Protected Method
        /// <summary>
        /// We need to know if the last text changed event was due to one of the dropdowns 
        /// or to the keyboard
        /// </summary>
        /// <param name="e"></param>
        protected override void OnDropDownClosed(EventArgs e)
        {
            mFromKeyboard = false;
            base.OnDropDownClosed(e);
        }

        /// <summary>
        /// this were we can make suggestions
        /// </summary>
        /// <param name="e"></param>
        protected override void OnTextChanged(EventArgs e)
        {
            base.OnTextChanged(e);

            if (!mFromKeyboard || !Focused)
                return;

            mSuggestionList.BeginUpdate();
            mSuggestionList.Items.Clear();

            using (SearchStringMatcher lStringMatcher = new SearchStringMatcher(MatchingType, Text))
            {
                foreach (object lItem in Items)
                {
                    SearchMatchdInfo lSearchMatch = lStringMatcher.Match(GetItemText(lItem));
                    if (lSearchMatch != null)
                    {
                        mSuggestionList.Items.Add(lSearchMatch);
                    }
                }
            }

            mSuggestionList.EndUpdate();

            bool lVisible = mSuggestionList.Items.Count != 0;

            if (mSuggestionList.Items.Count == 1 && ((SearchMatchdInfo)mSuggestionList.Items[0]).Text.Length == Text.Trim().Length)
            {
                SearchMatchdInfo lSelection = (SearchMatchdInfo)mSuggestionList.Items[0];
                Text = lSelection.Text;
                Select(0, Text.Length);
                lVisible = false;
            }

            if (lVisible)
            {
                ShowDropDown();
            }
            else
            {
                HideDropDown();
            }

            mFromKeyboard = false;
        }

        /// <summary>
        /// Process command keys
        /// </summary>
        protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
        {
            if ((keyData == Keys.Tab) && (mCustomDropDown.Visible))
            {
                // we change the selection but will also allow the navigation to the next control
                if (mSuggestionList.Text.Length != 0)
                {
                    Text = mSuggestionList.Text;
                }

                Select(0, Text.Length);
                HideDropDown();
            }

            return base.ProcessCmdKey(ref msg, keyData);
        }

        /// <summary>
        /// if the dropdown is visible some keystrokes should behave in a custom way
        /// </summary>
        protected override void OnKeyDown(KeyEventArgs e)
        {
            mFromKeyboard = true;

            if (!mCustomDropDown.Visible)
            {
                if (e.KeyCode == Keys.Enter)
                {
                    //This Enter keydown event must be considered same as the event that occurs when the user selects an item in the suggestion list and hits the Enter key.
                    mFromKeyboard = false;
                }
                else
                {
                    base.OnKeyDown(e);
                    return;
                }
            }

            switch (e.KeyCode)
            {
                case Keys.Down:
                    if (mSuggestionList.SelectedIndex < 0)
                    {
                        mSuggestionList.SelectedIndex = 0;
                    }
                    else if (mSuggestionList.SelectedIndex < mSuggestionList.Items.Count - 1)
                    {
                        mSuggestionList.SelectedIndex++;
                    }
                    break;

                case Keys.Up:
                    if (mSuggestionList.SelectedIndex > 0)
                    {
                        mSuggestionList.SelectedIndex--;
                    }
                    else if (mSuggestionList.SelectedIndex < 0)
                    {
                        mSuggestionList.SelectedIndex = mSuggestionList.Items.Count - 1;
                    }
                    break;

                case Keys.Enter:
                    if (mSuggestionList.Text.Length != 0)
                    {
                        Text = mSuggestionList.Text;
                    }

                    Select(0, Text.Length);
                    HideDropDown();
                    //Raise item selection change event here
                    RaiseSearchItemSelectedEvent();
                    break;

                case Keys.Escape:
                    HideDropDown();
                    break;

                default:
                    base.OnKeyDown(e);
                    return;
            }

            e.Handled = true;
            e.SuppressKeyPress = true;
        }

        /// <summary>
        /// override the WndProc to draw the control border
        /// </summary>
        /// <param name="Msg"></param>
        protected override void WndProc(ref Message Msg)
        {
            base.WndProc(ref Msg);

            if (mBorderDrawer != null && DrawBorder )
                mBorderDrawer.DrawBorder(ref Msg, this.Width, this.Height);
        }

        /// <summary>
        /// <see cref="ComboBox.OnLostFocus(EventArgs)"/>
        /// </summary>
        protected override void OnLostFocus(EventArgs e)
        {
            if (!mCustomDropDown.Focused && !mSuggestionList.Focused)
            {
                HideDropDown();
            }

            base.OnLostFocus(e);
        }

        /// <summary>
        /// <see cref="ComboBox.OnDropDown(EventArgs)"/>
        /// </summary>
        protected override void OnDropDown(EventArgs e)
        {
            HideDropDown();
            base.OnDropDown(e);
        }

        /// <summary>
        /// Override the On location changed method
        /// </summary>
        /// <param name="e"></param>
        protected override void OnLocationChanged(EventArgs e)
        {
            base.OnLocationChanged(e);
            HideDropDown();
        }

        /// <summary>
        /// override the onsizeChanged method of the control
        /// </summary>
        /// <param name="e"></param>
        protected override void OnSizeChanged(EventArgs e)
        {
            base.OnSizeChanged(e);

            mCustomDropDown.Width = Width;
        }
        #endregion

        #region Private Methods
        /// <summary>
        /// We keep track of system settings changes for the font
        /// </summary>
        /// <param name="Sender"></param>
        /// <param name="e"></param>
        private void OnFontChanged(object Sender, EventArgs e)
        {
            if (mBoldFont != null)
            {
                mBoldFont.Dispose();
            }

            mSuggestionList.Font = Font;
            mBoldFont = new Font(Font, FontStyle.Underline);
            mSuggestionList.ItemHeight = mBoldFont.Height + 2;
        }

        /// <summary>
        /// Draw a segment of a string and updates the bound rectangle for being used for the next segment drawing
        /// </summary>
        /// <param name="GraphicsObject"></param>
        /// <param name="TextColor"></param>
        /// <param name="DrawingRectangle"></param>
        /// <param name="Text"></param>
        /// <param name="RequiredFont"></param>
        private static void DrawString(Graphics GraphicsObject, Color TextColor, ref Rectangle DrawingRectangle, string Text, Font RequiredFont)
        {
            Size LProposedSize = new Size(int.MaxValue, int.MaxValue);
            Size lSize = TextRenderer.MeasureText(GraphicsObject, Text, RequiredFont, LProposedSize, TextFormatFlags.NoPadding);
            TextRenderer.DrawText(GraphicsObject, Text, RequiredFont, DrawingRectangle, TextColor, TextFormatFlags.Left | TextFormatFlags.VerticalCenter | TextFormatFlags.NoPadding | TextFormatFlags.NoPrefix);
            DrawingRectangle.X += lSize.Width;
            DrawingRectangle.Width -= lSize.Width;
        }

        /// <summary>
        /// Draw an item in the suggestion listbox
        /// </summary>
        /// <param name="Sender"></param>
        /// <param name="e"></param>
        private void OnSuggestionListDrawItem(object Sender, DrawItemEventArgs e)
        {
            SearchMatchdInfo lSearchMatch = (SearchMatchdInfo)mSuggestionList.Items[e.Index];

            e.DrawBackground();

            bool lIsBold = lSearchMatch.StartsOnMatch;
            Rectangle rBounds = e.Bounds;

            foreach (string lString in lSearchMatch.Segments)
            {
                DrawString(e.Graphics, ForeColor, ref rBounds, lString, lIsBold ? mBoldFont : Font);
                lIsBold = !lIsBold;
            }

            e.DrawFocusRectangle();
        }

        /// <summary>
        /// We highlight the selection under the mouse in the suggestion listbox
        /// </summary>
        /// <param name="Sender"></param>
        /// <param name="e"></param>
        private void OnSuggestionListMouseMove(object Sender, MouseEventArgs e)
        {
            int lIndex = mSuggestionList.IndexFromPoint(e.Location);
            if ((lIndex >= 0) && (lIndex != mSuggestionList.SelectedIndex))
            {
                mSuggestionList.SelectedIndex = lIndex;
            }

            Invalidate();
        }

        /// <summary>
        /// Called when the user clicks on an item in the list
        /// </summary>
        private void OnSuggestionListClick(object sender, EventArgs e)
        {
            mFromKeyboard = false;
            SearchMatchdInfo mSel = (SearchMatchdInfo)mSuggestionList.SelectedItem;
            Text = mSel.Text;
            Select(0, Text.Length);
            Focus();
            //Raise Item selection change event here
            RaiseSearchItemSelectedEvent();
        }

        /// <summary>
        /// Raise search Item selected event to notify the consumer      
        /// </summary>
        private void RaiseSearchItemSelectedEvent()
        {
            if (SearchItemSelected == null)
                return;

            SearchTextArgs lSearchTextArgs = new SearchTextArgs();
            lSearchTextArgs.SearchText = Text;
            Text = string.Empty;
            SearchItemSelected(lSearchTextArgs);            
        }

        /// <summary>
        /// Set the basic propertes for the required control
        /// </summary>
        private void SetBasicTheme()
        {
            Font = new Font("Microsoft Sans Serif", 9, FontStyle.Regular);
            ForeColor = ColorTranslator.FromHtml("#666666");
            // we're overriding these
            DropDownStyle = ComboBoxStyle.DropDown;
            AutoCompleteMode = AutoCompleteMode.None;
        }

        /// <summary>
        /// Method to set the cotrol floating text by sending PInvoke message
        /// </summary>
        private void UpdateFloatingText()
        {
            if (!IsHandleCreated || string.IsNullOrEmpty(mFloatingText))
                return;

            IntPtr lEditControlHandle = GetEditHandle(this);
            if (lEditControlHandle != IntPtr.Zero)
            {
                IntPtr lMem = Marshal.StringToHGlobalUni(mFloatingText);
                SendMessage(lEditControlHandle, 0x1501, (IntPtr)1, lMem);
                Marshal.FreeHGlobal(lMem);
            }
        }

        /// <summary>
        /// Get the editbox control's handle from the combobox
        /// </summary>
        /// <param name="control"></param>
        /// <returns></returns>
        private IntPtr GetEditHandle(ComboBox control)
        {
            COMBOBOXINFO info = new COMBOBOXINFO();
            info.CbSize = Marshal.SizeOf(info);

            return GetComboBoxInfo(control.Handle, ref info) ? info.HwndEdit : IntPtr.Zero;
        }

        /// <summary>
        /// Override the OnHandleCreated method to send message for binding the
        /// floating text to the control
        /// </summary>
        /// <param name="e"></param>
        protected override void OnHandleCreated(EventArgs e)
        {
            base.OnHandleCreated(e);
            UpdateFloatingText();
        }
        #endregion

        #region Dispose Object
        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        /// <param name="Disposing"></param>
        protected override void Dispose(bool Disposing)
        {
            if (mDisposed)
                return;

            if (Disposing)
            {
                mSafeHandle.Dispose();
                mSafeHandle = null;
                FontChanged -= OnFontChanged;

                // Free any other managed objects here.
                if (mBoldFont != null)
                {
                    mBoldFont.Dispose();
                    mBoldFont = null;
                }

                if (mCustomDropDown != null)
                {
                    mCustomDropDown.Dispose();
                    mCustomDropDown = null;
                }

                if (mSuggestionList != null)
                {
                    mSuggestionList.Click -= OnSuggestionListClick;
                    mSuggestionList.DrawItem -= OnSuggestionListDrawItem;
                    mSuggestionList.MouseMove -= OnSuggestionListMouseMove;
                    mSuggestionList.Dispose();
                    mSuggestionList = null;
                }

                SearchItemSelected = null;

                mBorderDrawer = null;
            }

            // Free any unmanaged objects here.below

            //set the value to indicate the object has been disposed
            mDisposed = true;

            // Call base class implementation.
            base.Dispose(Disposing);
        }
        #endregion
    }    
}
