﻿using System;
using System.Text;
using System.Data;
using System.Linq;
using System.Drawing;
using System.Collections;
using System.Collections.Generic;
using System.Windows.Forms;
using System.ComponentModel;
using System.Runtime.InteropServices;

namespace Eka.Inheritance.MDIControls
{
    /// <summary>
    /// Drop down list with multiple selections, rather than the 
    /// standard one select and out.
    /// </summary>
    [ToolboxItem(true)]
    public partial class MultiSelectDropList_Old : MultiSelectDropListCheckedListBox
    {
        public string mSelectedValues = null;
        private DataTable mDataTable = null;
        private string mDisplayMember = string.Empty;
        private string mValueMember = string.Empty;

        public enum Separator
        { Comma, Hyphen, SemiColon };

        /// <summary>
        /// A tooltip to show the full text when abbreviated.
        /// </summary>
        ToolTip mToolTip = new ToolTip();

        #region Properties
        /// <summary>
        /// The items for the list
        /// </summary>
        [EditorBrowsable(EditorBrowsableState.Never)]
        public ListBox.ObjectCollection Items
        {
            get { return Content.Items; }
        }

        [EditorBrowsable(EditorBrowsableState.Never)]
        public CheckedListBox.CheckedItemCollection CheckedItems
        {
            get { return Content.CheckedItems; }
        }

        [EditorBrowsable(EditorBrowsableState.Never)]
        public string SelectedValues
        {
            get { return mSelectedValues; }
            set
            {
                mSelectedValues = value;
            }
        }

        [EditorBrowsable(EditorBrowsableState.Never)]
        public DataTable DataSource
        {
            get { return mDataTable; }
            set
            {
                mDataTable = value;
            }
        }

        [EditorBrowsable(EditorBrowsableState.Never)]
        public string DisplayMember
        {
            get { return mDisplayMember; }
            set { mDisplayMember = value; }
        }

        [EditorBrowsable(EditorBrowsableState.Never)]
        public string ValueMember
        {
            get { return mValueMember; }
            set { mValueMember = value; }
        }

        [EditorBrowsable(EditorBrowsableState.Never)]
        public Separator mSeparator
        {
            get;
            set;
        }

        #endregion

        public MultiSelectDropList_Old()
        {
            InitializeComponent();
            Content.SelectionMode = SelectionMode.One;
            Content.CheckOnClick = true;
            Content.ItemCheck += content_ItemCheck;
            Content.SelectedIndexChanged += new EventHandler(content_SelectedIndexChanged);

            // Set up the delays for the ToolTip.
            mToolTip.AutoPopDelay = 5000;
            mToolTip.InitialDelay = 1000;
            mToolTip.ReshowDelay = 500;

            // Force the ToolTip text to be displayed whether or not the form is active.
            mToolTip.ShowAlways = true;
        }

        #region Public Methods
        public void EkaDisplay(DataTable SourceTable, string DisplayMember, string ValueMember, bool AllowAll, Separator Separator)
        {
            int lTotalRecords = 0;

            ctrButton.Text = string.Empty;
            Content.Items.Clear();

            if (SourceTable != null && SourceTable.Rows.Count > 0)
            {
                if (AllowAll == true)
                {
                    DataRow dr = SourceTable.NewRow();
                    dr[0] = "<All>";
                    SourceTable.Rows.InsertAt(dr, 0);
                }
            }

            this.DataSource = SourceTable;
            this.DisplayMember = DisplayMember;
            this.ValueMember = ValueMember;
            this.mSeparator = Separator;

            if (DataSource != null && DataSource.Rows.Count > 0)
            {
                lTotalRecords = DataSource.Rows.Count;
                object[] lDisplayValue = new object[lTotalRecords];

                for (int i = 0; i < lTotalRecords; i++)
                {
                    lDisplayValue[i] = DataSource.Rows[i][DisplayMember];
                }
                Content.Items.AddRange(lDisplayValue);
            }

            Content.MaximumSize = new Size(ctrButton.Width, 100);
            Content.MinimumSize = new Size(ctrButton.Width, 100);            
        }
        #endregion

        #region Private Methods
        /// <summary>
        /// Set the control text, fitting it as needed.
        /// </summary>
        /// <param name="text"></param>
        private void SetDropDownText(string text)
        {
            // Tooltip: the list itself
            mToolTip.SetToolTip(ctrButton, text.Replace("; ", "\n"));

            // The button gets an abbreviated version.
            text = FitTextToSpace(text,
                                       Width - 25,
                                       2,
                                       string.Format("++ {0} Items ++", Content.CheckedItems.Count),
                                       Font);
            ctrButton.Text = text;
            mToolTip.BackColor = System.Drawing.Color.Red;
        }

        /// <summary>
        /// Fit the text in the space
        /// If there is not enough space, first truncate and add an elipsis,
        /// Then replace with the substitution string when it exceeds the limit
        /// </summary>
        /// <param name="text"></param>
        /// <param name="availableSpace"></param>
        /// <param name="limitMultiplier"></param>
        /// <param name="substituteText"></param>
        /// <param name="font"></param>
        /// <returns></returns>
        private static string FitTextToSpace(string text, int availableSpace, int limitMultiplier, string substituteText, Font font)
        {
            Size s = TextRenderer.MeasureText(text, font);

            if (s.Width > availableSpace)
            {
                if (s.Width > availableSpace * limitMultiplier && !string.IsNullOrEmpty(substituteText))
                {
                    text = substituteText;
                }
                else
                {
                    while (s.Width > availableSpace)
                    {
                        text = text.Substring(0, text.Length - 1);
                        s = TextRenderer.MeasureText(text + "...", font);
                    }
                    text += "...";
                }
            }
            return text;
        }

        #endregion

        #region Events
        /// <summary>
        /// Event to indicate Selected Items changed
        /// </summary>
        public event EventHandler EkaSelectedValueChanged;

        public event MouseEventHandler MouseDown;

        public void EkaSetTemplateValues(string SelectedIDs)
        {
            List<string> lSelectedValues = new List<string>();
            StringBuilder lTextToDiplayInDropDown = new StringBuilder();
            string lSeparatorForIDs = ",";
            string lSeparatorForValues = ";";

            if (mSeparator == Separator.Comma)
            {
                lSeparatorForValues = ",";
            }

            if (mSeparator == Separator.Hyphen)
            {
                lSeparatorForValues = "-";
            }

            if (mSeparator == Separator.SemiColon)
            {
                lSeparatorForValues = ";";
            }

            if(!string.IsNullOrEmpty(SelectedIDs))
            {
                if (DataSource != null && DataSource.Rows.Count > 0) {
                    string[] lItems = SelectedIDs.Split(lSeparatorForIDs.ToCharArray());

                    for (int i = 0; i < lItems.Length; i++)
                    {
                        DataRow[] lDatarow = DataSource.Select("" + mValueMember + " = " + lItems[i].ToString());
                        string lSelectedValue = string.Empty;

                        if(lDatarow.Count() > 0 )
                        {
                            lSelectedValue = lDatarow[0][mDisplayMember].ToString();
                        }

                        if (!string.IsNullOrEmpty(lSelectedValue))
                        {
                            lTextToDiplayInDropDown.AppendFormat("{0}{1}", lSelectedValue, lSeparatorForValues);

                            for (int j = 1; j < Content.Items.Count; j++)
                            {
                                if (Content.Items[j].ToString() == lSelectedValue)
                                {
                                    Content.SetItemChecked(j, true);
                                    break;
                                }
                            }
                        }
                    }
                }

                if (lTextToDiplayInDropDown.Length > 0)
                {
                    lTextToDiplayInDropDown.Length = lTextToDiplayInDropDown.Length--;
                }
                
                SetDropDownText(lTextToDiplayInDropDown.ToString());
            }
        }

        /// <summary>
        /// Called to signal to subscribers that Selected Items changed
        /// </summary>
        /// <param name="e"></param>
        protected virtual void EkaOnSelectedValueChanged(EventArgs e)
        {
            EventHandler eh = EkaSelectedValueChanged;
            if (eh != null)
            {
                eh(this, e);
            }
        }

        protected virtual void OnMouseDown(object sender, MouseEventArgs e) {
            MouseEventHandler eh = MouseDown;
            if (eh != null) {
                eh(this, e);
            }
        }

        /// <summary>
        /// Clicked - open the drop down
        /// It will close when the user clicks away from it.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ctrButton_Click(object sender, EventArgs e)
        {
            Open();
        }

        /// <summary>
        /// Combo selection changed
        /// Update drop down text, and pass event on up.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void content_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (Content.SelectedItems.Count == 0)
            {
                //Do Nothing
            }
            else
            {
                StringBuilder lTextToDiplayInDropDown = new StringBuilder();
                StringBuilder lSelectedItemIds = new StringBuilder();
                string lSeparatorForValues = string.Empty;
                string lSeparatorForIDs = string.Empty;

                foreach (object ItemsObject in Content.CheckedItems)
                {

                    lTextToDiplayInDropDown.AppendFormat("{0}{1}", lSeparatorForValues, ItemsObject.ToString());

                    object lSelectedId = (from DataRow dr in DataSource.Rows
                                          where (string)dr[DisplayMember] == ItemsObject.ToString()
                                          select (object)dr[ValueMember]).FirstOrDefault();

                    lSelectedItemIds.AppendFormat("{0}{1}", lSeparatorForIDs, lSelectedId.ToString());

                    // As IDs are being used in SQL queries so let them be separated by Comma
                    lSeparatorForIDs = ",";
                    if (mSeparator == Separator.Comma)
                    {
                        lSeparatorForValues = ",";
                    }

                    if (mSeparator == Separator.Hyphen)
                    {
                        lSeparatorForValues = "-";
                    }

                    if (mSeparator == Separator.SemiColon)
                    {
                        lSeparatorForValues = ";";
                    }
                }

                SelectedValues = lSelectedItemIds.ToString();

                if (lTextToDiplayInDropDown.ToString().Contains("All"))
                {
                    if (Content.CheckedItems.Count > 1)
                    { lTextToDiplayInDropDown.Remove(0, 6); }
                    else
                    { lTextToDiplayInDropDown.Remove(0, 5); }

                    SelectedValues = SelectedValues.Remove(0, 1);
                }

                SetDropDownText(lTextToDiplayInDropDown.ToString());
            }
            EkaOnSelectedValueChanged(null);
        }

        /// <summary>
        /// 'Select All' behaviour implemented
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void content_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            Content.ItemCheck -= content_ItemCheck;
            if (Content.Items.Contains("<All>"))
            {
                if (e.Index == 0 && e.NewValue == CheckState.Unchecked)
                {
                    for (int i = 1; i < Content.Items.Count; i++)
                    {
                        Content.SetItemChecked(i, false);
                    }
                }

                if (e.Index == 0 && e.NewValue == CheckState.Checked)
                {

                    for (int i = 1; i < Content.Items.Count; i++)
                    {
                        Content.SetItemChecked(i, true);
                    }
                }

                if (e.Index != 0 && e.NewValue == CheckState.Unchecked)
                {
                    Content.SetItemChecked(0, false);
                }

                if (e.Index != 0 && e.NewValue == CheckState.Checked)
                {
                    int j = 0;
                    for (int i = 1; i < Content.Items.Count; i++)
                    {
                        if (Content.GetItemChecked(i))
                        {
                            j += 1;
                        }
                        if (j == (Content.Items.Count - 2))
                        {
                            Content.SetItemChecked(0, true);
                        }
                    }
                }
            }
            Content.ItemCheck += content_ItemCheck;
        }

        private void ctrButton_MouseDown(object sender, MouseEventArgs e) {
            OnMouseDown(sender, e);
        }
        #endregion
    }
    /// <summary>
    /// This class exists solely to provide a non-abstract layer
    /// </summary>
    [ToolboxItem(false)]
    public class MultiSelectDropListCheckedListBox : ctrPopUp<CheckedListBox> { }
}